// input expression levels and statistical test resutls.
#include "Input.h"

GeneData::GeneData(void){
  tracking_id = "-";
  cluster = 0;
}

GeneData::GeneData(int num){
  for(int i = 0; i < num; ++i){ test.push_back(WILD_CARD); }
  tracking_id = "-";
  cluster = 0;
}

GeneData::~GeneData(void){
  vector<double>().swap(exp);
  vector<int>().swap(test);
}

void GeneData::init_test(int num){
  for(int i = 0; i < num; ++i){ test.push_back(0); }
}

bool GeneData::sign_test(void){
  vector<int>::iterator it;
  for(it = test.begin(); it != test.end(); ++it){
    if(*it != WILD_CARD){ return true; }
  }
  return false;
}

bool GeneData::sign_exp(void){
  vector<double>::iterator it;
  for(it = exp.begin(); it != exp.end(); ++it){
    if(*it != 0){ return true; }
  }
  return false;
}

unsigned int calc_dim(const int stage, const unsigned int sample1_num, const unsigned int sample2_num){
  return (unsigned int)((double)(stage - 1.5 - 0.5 * sample1_num) * sample1_num + sample2_num - 1);
}

void input_diff(map<string, GeneData*> *gene_list, vector<string> &labels, const int stage, const string diff_file, const double FDR){
  ifstream diffFile(diff_file.c_str());
  string get_input, id, sample1_name, sample2_name;
  string delim(" \t");
  double logarithm(0), qval(0);
  vector<string> line;
  vector<string>::iterator it;
  unsigned int i(0), sample1_num(0), sample2_num(0), dim(0);

  if(diffFile.fail()){
    cerr << "ERROR: Can't open diff_file." << endl;
    exit(1);
  }

  getline(diffFile, get_input);
  while(diffFile && getline(diffFile, get_input)){
    boost::split(line, get_input, boost::is_any_of(delim));
    logarithm = 0;

    id = line[TRACKING_ID];
    sample1_name = line[SAMPLE_1];
    sample2_name = line[SAMPLE_2];
    logarithm = atof(line[LOGFC].c_str());
    qval = atof(line[QVAL].c_str());

    if(qval < FDR){
      for(i = 0; i < labels.size(); ++i){
	if(labels[i] == sample1_name){ sample1_num = i; }
	else if(labels[i] == sample2_name){ sample2_num = i; }
      }
      dim = calc_dim(stage, sample1_num, sample2_num);
      if(gene_list->find(id) == gene_list->end()){
	cerr << "ERROR: diff file contains genes which were not registered in tracking file." << endl;
	exit(1);
      }else{
	if(logarithm > 0){ (*gene_list)[id]->edit_test(dim, UP); }
	else if(logarithm < 0){ (*gene_list)[id]->edit_test(dim, DOWN); }
	else{
	  cerr << "ERROR: qval < FDR, but log2(fold_change) == 0" << endl;
	  exit(1);
	}
      }
    }
  }
}

void input_expression_levels(map<string, GeneData*> *gene_list, vector<string> &labels, const int test_num, const string tracking_file){
  ifstream trackingFile(tracking_file.c_str());
  GeneData *data;
  string get_input, id;
  string delim(" \t");
  vector<string> line;
  vector<string>::iterator it;

  if(trackingFile.fail()){
    cerr << "ERROR: Can't open tracking_file." << endl;
    exit(1);
  }

  //get sample labels.
  getline(trackingFile, get_input);
  boost::split(line, get_input, boost::is_any_of(delim));
  for(it = line.begin() + 1; it != line.end(); ++it){ labels.push_back(*it); }

  //get each gene info.
  while(trackingFile && getline(trackingFile, get_input)){
    data = new GeneData(test_num);
    boost::split(line, get_input, boost::is_any_of(delim));
    id = line[TRACKING_ID];
    data->set_tracking_id(id);
    for(it = line.begin() + 1; it != line.end(); ++it){ data->push_exp(atof((*it).c_str()));}
    gene_list->insert(make_pair(id, data));
  }
}

void input(map<string, GeneData*> *gene_list, vector<string> &labels, const int stage, const string tracking_file, const string diff_file, const double FDR){
  int test_num = stage * (stage - 1) / 2;
  input_expression_levels(gene_list, labels, test_num, tracking_file);
  input_diff(gene_list, labels, stage, diff_file, FDR);
}
