// input expression levels and statistical test resutls.
#ifndef INPUT_H
#define INPUT_H
#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <vector>
#include <cstring>
#include <cstdlib>
#include <boost/foreach.hpp>
#include <boost/algorithm/string.hpp>
#include <omp.h>

// field number of diff file.
#define TRACKING_ID 0
#define SAMPLE_1 1
#define SAMPLE_2 2
#define LOGFC 3
#define QVAL 4

// test results.
#define UP 1
#define DOWN -1
#define WILD_CARD 99

using namespace std;

class GeneData{
 protected:
  string tracking_id;
  int cluster;
  vector<double> exp;
  vector<int> test;

 public:
  GeneData(void);
  GeneData(int num);
  ~GeneData(void);
  void init_test(int num);
  void set_tracking_id(string str){ tracking_id = str; }
  void set_cluster(int val){ cluster = val; }
  void set_exp(vector<double> val){ exp = val; }
  void set_test(vector<int> val){ test = val; }
  void push_exp(double val){ exp.push_back(val); }
  void push_test(int val){ test.push_back(val); }
  void edit_test(int dim, int val){ test[dim] = val; }

  string get_tracking_id(void) const { return tracking_id; }
  int get_cluster(void) const { return cluster; }
  vector<double> get_exp(void) const { return exp; }
  vector<int> get_test(void) const { return test; }
  int size_exp(void) const { return exp.size(); }
  int size_test(void) const { return test.size(); }
  bool sign_test(void);
  bool sign_exp(void);
};

unsigned int calc_dim(const int stage, const unsigned int sample1_num, const unsigned int sample2_num);

void input_diff(map<string, GeneData*> *gene_list, vector<string> &labels, const int stage, const string diff_file, const double FDR);

void input_expression_levels(map<string, GeneData*> *gene_list, vector<string> &labels, const int test_num, const string tracking_file);

void input(map<string, GeneData*> *gene_list, vector<string> &labels, const int stage, const string tracking_file, const string diff_file, const double FDR);

#endif
