// Main function.
// developer : aoto
// date      : 2017 Nov.

#include <iostream>
#include <string>
#include <map>
#include <getopt.h>
#include <cstdlib>
#include <omp.h>
#include "Input.h"
#include "Clustering.h"
#include "Methods.h"
#include "Output.h"

using namespace std;

void Help(void){
  cout << endl;
  cout << "program." << endl;
  cout << "name         : DEclust" << endl;
  cout << "version      : 1.0.1" << endl;
  cout << "editing      : Nov. 2017" << endl;
  cout << endl;
  cout << "usage." << endl;
  cout << " DEclust [options] -n <number_of_conditions> -d <input.statistical_tests> -t <input.exp>" << endl;
  cout << endl;
  cout << "main arguments." << endl;
  cout << " -n <int>    : Number of conditions (samples)." << endl;
  cout << " -d <string> : List of differentially expressed transcriptional units." << endl;
  cout << " -t <string> : Tracking file of gene expression levels." << endl;
  cout << endl;
  cout << "options." << endl;
  cout << " -h          : Display usage." << endl;
  cout << " -a          : If set, all genes that are expressed at least one condition are clustered.     (default : only DEGs are clustered)" << endl;
  cout << " -D <float>  : Distance definition (cosine, euclidean, pearson).                              (default : cosine)" << endl;
  cout << " -m <string> : Merge method (average, single, complete, ward).                                (default : average)" << endl;
  cout << " -T <float>  : Distance threshold for merging clusters.                                       (default : 1.0)" << endl;
  cout << " -f <float>  : FDR threshold for DE detection.                                                (default : 0.05)" << endl;
  cout << " -p <int>    : Num. of threads.                                                               (default : 1)" << endl;
  cout << " -o <string> : Prefix of output files.                                                        (default : ./clustering_results)" << endl;
  cout << " -e          : If set, output a belonging cluster for each iteration for evaluation." << endl;
  cout << endl;
  exit(1);
}

int main(int argc, char *argv[]){
  int args, stage = 0, core = 1;
  double threshold = 1.0, FDR = 0.05;
  string opt_tmp, tracking_file, diff_file, method = "average", distance = "cosine", output_prefix = "./clustering_results";
  bool DEGs = true, eval = false;

  while((args = getopt(argc, argv, "han:t:d:D:m:T:f:p:o:e")) != -1){
    switch(args){
    case 'h':
      Help();
      break;
    case 'n':
      opt_tmp = (string)optarg;
      stage = atoi(opt_tmp.c_str());
      break;
    case 't':
      tracking_file = (string)optarg;
      break;
    case 'd':
      diff_file = (string)optarg;
      break;
    case 'a':
      DEGs = false;
      break;
    case 'T':
      opt_tmp = (string)optarg;
      threshold = atof(opt_tmp.c_str());
      break;
    case 'm':
      method = (string)optarg;
      break;
    case 'D':
      distance = (string)optarg;
      break;
    case 'f':
      opt_tmp = (string)optarg;
      FDR = atof(opt_tmp.c_str());
      break;
    case 'p':
      opt_tmp = (string)optarg;
      core = atoi(opt_tmp.c_str());
      break; 
    case 'o':
      output_prefix = (string)optarg;
      break;
    case 'e':
      eval = true;
      break;
    case '?':
    default:
      cerr << "ERROR: Incorrect option." << endl;
      exit(1);
    }
  }

  if(argc == 1){
    Help();
  }else if(stage == 0 || tracking_file.length() == 0 || diff_file.length() == 0){
    cout << "ERROR: Main arguments are incorrect." << endl;
    Help();
    exit(1);
  }else{
    cout << "The definition of distance\t : " << distance << endl;
    cout << "Cluster merge method\t\t : " << method << endl;
    cout << "Cluster merge threshold\t\t : " << threshold << endl;

    omp_set_num_threads(core);
    MidNode *node;
    vector<string> labels;
    vector<vector<double> > exp_dist_table;
    map<string, vector<int> > log;
    map<string, GeneData*> *gene_list = new map<string, GeneData*>;
    vector<MidNode *> *entry = new vector<MidNode *>;
    input(gene_list, labels, stage, tracking_file, diff_file, FDR);
    Preprocessing(gene_list, entry, exp_dist_table, DEGs, distance, core);
    node = cluster_analysis(entry, exp_dist_table, log, method, threshold, eval, core);
    Output(node, labels, log, output_prefix, stage, eval);
  }
  return 0;
}
