// definition of clustering methods.
#include "Methods.h"

Average::Average(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const double t, const bool eval, const int core) : Clustering(entry, exp_dist_table, t, eval, core){
  if(eval == false){ clustering(entry, exp_dist_table); }
  else{ clustering_eval(entry, exp_dist_table); }
}

void Average::clustering(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    //update_second_node(entry, exp_dist_table);
    ++iteration_num;
  }
}

void Average::clustering_eval(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    logging(entry);
    ++iteration_num;
  }
}

void Average::update_first_node(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  int i(0);
  Similarity *tmp_sim = NULL;
  vector<int>::iterator it;
  const vector<int>::iterator it_f = find(search_list.begin(), search_list.end(), first_node);

  // update similarity table and exp_dist_table.
  for(it = it_f + 1; it != search_list.end(); ++it){
    i = (*it);
    if(i != second_node){
      exp_dist_table[first_node][i] = calc_exp_dist(exp_dist_table, i);
      //exp_dist_table[first_node][i] = calc_exp_dist(entry, exp_dist_table, i);
      exp_dist_table[i][first_node] = exp_dist_table[first_node][i];
      tmp_sim = table[first_node][i];
      table[first_node][i] = new Similarity((*entry)[first_node], (*entry)[i], exp_dist_table[first_node][i]);
      delete tmp_sim;
    }
  }
  for(it = search_list.begin(); it != it_f; ++it){
    i = (*it);
    exp_dist_table[i][first_node] = calc_exp_dist(exp_dist_table, i);
    //exp_dist_table[i][first_node] = calc_exp_dist(entry, exp_dist_table, i);
    exp_dist_table[i][first_node] = exp_dist_table[first_node][i];
    tmp_sim = table[i][first_node];
    table[i][first_node] = new Similarity((*entry)[i], (*entry)[first_node], exp_dist_table[i][first_node]);
    delete tmp_sim;
  }
}

//inline double Average::calc_exp_dist(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const int &obj){
inline double Average::calc_exp_dist(vector<vector<double> > &exp_dist_table, const int &obj){
  double exp_dist = ((double)first_node_size / (first_node_size + second_node_size) * exp_dist_table[first_node][obj]) + ((double)second_node_size / (first_node_size + second_node_size) * exp_dist_table[second_node][obj]);
  return exp_dist;
}

Single::Single(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const double t, const bool eval, const int core) : Clustering(entry, exp_dist_table, t, eval, core){
  if(eval == false){ clustering(entry, exp_dist_table); }
  else{ clustering_eval(entry, exp_dist_table); }
}

void Single::clustering(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    ++iteration_num;
  }
}

void Single::clustering_eval(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    logging(entry);
    ++iteration_num;
  }
}

void Single::update_first_node(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  int i(0);
  Similarity *tmp_sim = NULL;
  vector<int>::iterator it;
  const vector<int>::iterator it_f = find(search_list.begin(), search_list.end(), first_node);

  // update similarity table and exp_dist_table.
  for(it = it_f + 1; it != search_list.end(); ++it){
    i = (*it);
    if(i != second_node){
      exp_dist_table[first_node][i] = calc_exp_dist(exp_dist_table, i);
      //exp_dist_table[first_node][i] = calc_exp_dist(entry, exp_dist_table, i);
      tmp_sim = table[first_node][i];
      table[first_node][i] = new Similarity((*entry)[first_node], (*entry)[i], exp_dist_table[first_node][i]);
      delete tmp_sim;
    }
  }
  for(it = search_list.begin(); it != it_f; ++it){
    i = (*it);
    exp_dist_table[i][first_node] = calc_exp_dist(exp_dist_table, i);
    //exp_dist_table[i][first_node] = calc_exp_dist(entry, exp_dist_table, i);
    tmp_sim = table[i][first_node];
    table[i][first_node] = new Similarity((*entry)[i], (*entry)[first_node], exp_dist_table[i][first_node]);
    delete tmp_sim;
  }
}

//inline double Single::calc_exp_dist(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const int &obj){
inline double Single::calc_exp_dist(vector<vector<double> > &exp_dist_table, const int &obj){
  if(exp_dist_table[first_node][obj] < exp_dist_table[second_node][obj]){ return exp_dist_table[first_node][obj]; }
  else{ return exp_dist_table[second_node][obj]; }
}

Complete::Complete(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const double t, const bool eval, const int core) : Clustering(entry, exp_dist_table, t, eval, core){
  if(eval == false){ clustering(entry, exp_dist_table); }
  else{ clustering_eval(entry, exp_dist_table); }
}

void Complete::clustering(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    ++iteration_num;
  }
}

void Complete::clustering_eval(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    logging(entry);
    ++iteration_num;
  }
}

void Complete::update_first_node(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  int i(0);
  Similarity *tmp_sim = NULL;
  vector<int>::iterator it;
  const vector<int>::iterator it_f = find(search_list.begin(), search_list.end(), first_node);

  // update similarity table and exp_dist_table.
  for(it = it_f + 1; it != search_list.end(); ++it){
    i = (*it);
    if(i != second_node){
      exp_dist_table[first_node][i] = calc_exp_dist(exp_dist_table, i);
      //exp_dist_table[first_node][i] = calc_exp_dist(entry, exp_dist_table, i);
      tmp_sim = table[first_node][i];
      table[first_node][i] = new Similarity((*entry)[first_node], (*entry)[i], exp_dist_table[first_node][i]);
      delete tmp_sim;
    }
  }
  for(it = search_list.begin(); it != it_f; ++it){
    i = (*it);
    exp_dist_table[i][first_node] = calc_exp_dist(exp_dist_table, i);
    //exp_dist_table[i][first_node] = calc_exp_dist(entry, exp_dist_table, i);
    tmp_sim = table[i][first_node];
    table[i][first_node] = new Similarity((*entry)[i], (*entry)[first_node], exp_dist_table[i][first_node]);
    delete tmp_sim;
  }
}

//inline double Complete::calc_exp_dist(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const int &obj){
inline double Complete::calc_exp_dist(vector<vector<double> > &exp_dist_table, const int &obj){
  if(exp_dist_table[first_node][obj] > exp_dist_table[second_node][obj]){ return exp_dist_table[first_node][obj]; }
  else{ return exp_dist_table[second_node][obj]; }
}

Ward::Ward(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const double t, const bool eval, const int core) : Clustering(entry, exp_dist_table, t, eval, core){
  if(eval == false){ clustering(entry, exp_dist_table); }
  else{ clustering_eval(entry, exp_dist_table); }
}

void Ward::clustering(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    ++iteration_num;
  }
}

void Ward::clustering_eval(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  while(iteration_num < (NUM - 1)){
    search_most_similar_nodes(entry);
    //search_most_similar_nodes(entry, exp_dist_table);
    merge(entry);
    update_first_node(entry, exp_dist_table);
    update_second_node(exp_dist_table);
    logging(entry);
    ++iteration_num;
  }
}

void Ward::update_first_node(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table){
  int i(0);
  Similarity *tmp_sim = NULL;
  vector<int>::iterator it;
  const vector<int>::iterator it_f = find(search_list.begin(), search_list.end(), first_node);

  // update similarity table and exp_dist_table.
  for(it = it_f + 1; it != search_list.end(); ++it){
    i = (*it);
    if(i != second_node){
      exp_dist_table[first_node][i] = calc_exp_dist(entry, exp_dist_table, i);
      tmp_sim = table[first_node][i];
      table[first_node][i] = new Similarity((*entry)[first_node], (*entry)[i], exp_dist_table[first_node][i]);
      delete tmp_sim;
    }
  }
  for(it = search_list.begin(); it != it_f; ++it){
    i = (*it);
    exp_dist_table[i][first_node] = calc_exp_dist(entry, exp_dist_table, i);
    tmp_sim = table[i][first_node];
    table[i][first_node] = new Similarity((*entry)[i], (*entry)[first_node], exp_dist_table[i][first_node]);
    delete tmp_sim;
  }
}

inline double Ward::calc_exp_dist(vector<MidNode *> *entry, vector<vector<double> > &exp_dist_table, const int &obj){
  unsigned int obj_node_size = (*entry)[obj]->get_node_size();
  double exp_dist = ((double)(first_node_size + obj_node_size) / (first_node_size + second_node_size + obj_node_size) * exp_dist_table[first_node][obj]) + ((double)(second_node_size + obj_node_size) / (first_node_size + second_node_size) * exp_dist_table[second_node][obj]) - ((double)obj_node_size / (first_node_size + second_node_size) * exp_dist_table[first_node][second_node]);
  return exp_dist;
}
