// output clustering results.
#include "Output.h"

Output::Output(const MidNode *node, const vector<string> &labels, map<string, vector<int> > &log, const string output_prefix, const int s, const bool eval){
  stage = s;
  output_newick = output_prefix + ".newick";
  output_genes = output_prefix + ".genes";
  output_log = output_prefix + ".log";
  output_clustering_result(node, labels);
  if(eval == true){ output_clustering_log(log); }
}

void Output::output_clustering_result(const MidNode *node, const vector<string> &labels){
  int i, j;
  list<GeneData*>::iterator it_gene;
  vector<double>::iterator it_exp;
  vector<int>::iterator it_test;
  newick_file.open(output_newick.c_str());
  genes_file.open(output_genes.c_str());
  genes = (*node).get_node();

  //output newick format txt.
  newick_file << (*node).get_newick_format() << ";" << endl;

  //output stats of genes.
  genes_file << "tracking_id";
  for(i = 0; i < stage; ++i){ genes_file << "\t" << "SAMPLE_" << labels[i]; }
  for(i = 0; i < stage - 1; ++i){
    for(j = i + 1; j < stage; ++j){
      genes_file << "\t" << "(" << labels[i] << ", " << labels[j] << ")";
    }
  }
  genes_file << "\t" << "cluster";
  genes_file << endl;

  for(it_gene = genes.begin(); it_gene != genes.end(); ++it_gene){
    genes_file << (*it_gene)->get_tracking_id();
    exp = (*it_gene)->get_exp();
    for(it_exp = exp.begin(); it_exp != exp.end(); ++it_exp){ genes_file << "\t" << (*it_exp); }
    test = (*it_gene)->get_test();
    for(it_test = test.begin(); it_test != test.end(); ++it_test){
      if((*it_test) == WILD_CARD){ genes_file << "\t" << 0; }
      else{ genes_file << "\t" << (*it_test); }
    }
    genes_file << "\t" << (*it_gene)->get_cluster();
    genes_file << endl;
  }
}

void Output::output_clustering_log(map<string, vector<int> > &log){
  vector<int> log_vec;
  map<string, vector<int> >::iterator it_log;
  vector<int>::iterator it;
  log_file.open(output_log.c_str());
  for(it_log = log.begin(); it_log != log.end(); ++it_log){
    log_file << (*it_log).first;
    log_vec = (*it_log).second;
    for(it = log_vec.begin(); it != log_vec.end(); ++it){ log_file << "\t" << (*it); }
    log_file << endl;
  }
}
